@@ -3,6 +3,7 @@
 #include "addtr_18_pixmap.h"
 #include "addwp_18_pixmap.h"
 #include "begintr_18_pixmap.h"
+#include "bing_maps_pixmap.h"
 #include "cursor_addtr_pixmap.h"
 #include "cursor_addwp_pixmap.h"
 #include "cursor_begintr_pixmap.h"
@@ -13,6 +14,7 @@
 #include "cursor_geozoom_pixmap.h"
 #include "cursor_iscissors_pixmap.h"
 #include "cursor_mapdl_pixmap.h"
+#include "cursor_route_finder_pixmap.h"
 #include "cursor_ruler_pixmap.h"
 #include "cursor_showpic_pixmap.h"
 #include "cursor_zoom_pixmap.h"
@@ -24,7 +26,9 @@
 #include "iscissors_18_pixmap.h"
 #include "mapdl_18_pixmap.h"
 #include "mover_22_pixmap.h"
+#include "route_finder_18_pixmap.h"
 #include "ruler_18_pixmap.h"
+#include "select_18_pixmap.h"
 #include "showpic_18_pixmap.h"
 #include "thumbnails_pixmap.h"
 #include "vikaggregatelayer_pixmap.h"
@@ -32,7 +36,6 @@
 #include "vikdemlayer_pixmap.h"
 #include "vikgeoreflayer_pixmap.h"
 #include "vikgpslayer_pixmap.h"
-#include "viking_icon_pixmap.h"
 #include "viking_pixmap.h"
 #include "vikmapslayer_pixmap.h"
 #include "viktrwlayer_pixmap.h"
