@@ -62,6 +62,9 @@ VikLayerInterface vik_aggregate_layer_interface = {
 
   (VikLayerFuncSublayerRenameRequest)   NULL,
   (VikLayerFuncSublayerToggleVisible)   NULL,
+  (VikLayerFuncSublayerTooltip)         NULL,
+  (VikLayerFuncLayerTooltip)            NULL,
+  (VikLayerFuncLayerSelected)           NULL,
 
   (VikLayerFuncMarshall)		aggregate_layer_marshall,
   (VikLayerFuncUnmarshall)		aggregate_layer_unmarshall,
@@ -73,10 +76,16 @@ VikLayerInterface vik_aggregate_layer_interface = {
   (VikLayerFuncWriteFileData)           NULL,
 
   (VikLayerFuncDeleteItem)              NULL,
+  (VikLayerFuncCutItem)                 NULL,
   (VikLayerFuncCopyItem)                NULL,
   (VikLayerFuncPasteItem)               NULL,
   (VikLayerFuncFreeCopiedItem)          NULL,
   (VikLayerFuncDragDropRequest)		aggregate_layer_drag_drop_request,
+
+  (VikLayerFuncSelectClick)             NULL,
+  (VikLayerFuncSelectMove)              NULL,
+  (VikLayerFuncSelectRelease)           NULL,
+  (VikLayerFuncSelectedViewportMenu)    NULL,
 };
 
 struct _VikAggregateLayer {
@@ -186,20 +195,21 @@ VikAggregateLayer *vik_aggregate_layer_new ()
 void vik_aggregate_layer_insert_layer ( VikAggregateLayer *val, VikLayer *l, GtkTreeIter *replace_iter )
 {
   GtkTreeIter iter;
+  VikLayer *vl = VIK_LAYER(val);
 
-  if ( VIK_LAYER(val)->realized )
+  if ( vl->realized )
   {
-    vik_treeview_insert_layer ( VIK_LAYER(val)->vt, &(VIK_LAYER(val)->iter), &iter, l->name, val, l, l->type, l->type, replace_iter );
+    vik_treeview_insert_layer ( vl->vt, &(vl->iter), &iter, l->name, val, l, l->type, l->type, replace_iter );
     if ( ! l->visible )
-      vik_treeview_item_set_visible ( VIK_LAYER(val)->vt, &iter, FALSE );
-    vik_layer_realize ( l, VIK_LAYER(val)->vt, &iter );
+      vik_treeview_item_set_visible ( vl->vt, &iter, FALSE );
+    vik_layer_realize ( l, vl->vt, &iter );
 
     if ( val->children == NULL )
-      vik_treeview_expand ( VIK_LAYER(val)->vt, &(VIK_LAYER(val)->iter) );
+      vik_treeview_expand ( vl->vt, &(vl->iter) );
   }
 
   if (replace_iter) {
-    GList *theone = g_list_find ( val->children, vik_treeview_item_get_pointer ( VIK_LAYER(val)->vt, replace_iter ) );
+    GList *theone = g_list_find ( val->children, vik_treeview_item_get_pointer ( vl->vt, replace_iter ) );
     val->children = g_list_insert ( val->children, l, g_list_position(val->children,theone)+1 );
   } else {
     val->children = g_list_append ( val->children, l );
@@ -210,16 +220,17 @@ void vik_aggregate_layer_insert_layer ( VikAggregateLayer *val, VikLayer *l, Gtk
 void vik_aggregate_layer_add_layer ( VikAggregateLayer *val, VikLayer *l )
 {
   GtkTreeIter iter;
+  VikLayer *vl = VIK_LAYER(val);
 
-  if ( VIK_LAYER(val)->realized )
+  if ( vl->realized )
   {
-    vik_treeview_add_layer ( VIK_LAYER(val)->vt, &(VIK_LAYER(val)->iter), &iter, l->name, val, l, l->type, l->type);
+    vik_treeview_add_layer ( vl->vt, &(vl->iter), &iter, l->name, val, l, l->type, l->type);
     if ( ! l->visible )
-      vik_treeview_item_set_visible ( VIK_LAYER(val)->vt, &iter, FALSE );
-    vik_layer_realize ( l, VIK_LAYER(val)->vt, &iter );
+      vik_treeview_item_set_visible ( vl->vt, &iter, FALSE );
+    vik_layer_realize ( l, vl->vt, &iter );
 
     if ( val->children == NULL )
-      vik_treeview_expand ( VIK_LAYER(val)->vt, &(VIK_LAYER(val)->iter) );
+      vik_treeview_expand ( vl->vt, &(vl->iter) );
   }
 
   val->children = g_list_append ( val->children, l );
@@ -229,9 +240,10 @@ void vik_aggregate_layer_add_layer ( VikAggregateLayer *val, VikLayer *l )
 void vik_aggregate_layer_move_layer ( VikAggregateLayer *val, GtkTreeIter *child_iter, gboolean up )
 {
   GList *theone, *first, *second;
-  vik_treeview_move_item ( VIK_LAYER(val)->vt, child_iter, up );
+  VikLayer *vl = VIK_LAYER(val);
+  vik_treeview_move_item ( vl->vt, child_iter, up );
 
-  theone = g_list_find ( val->children, vik_treeview_item_get_pointer ( VIK_LAYER(val)->vt, child_iter ) );
+  theone = g_list_find ( val->children, vik_treeview_item_get_pointer ( vl->vt, child_iter ) );
 
   g_assert ( theone != NULL );
 
@@ -278,18 +290,19 @@ void vik_aggregate_layer_draw ( VikAggregateLayer *val, gpointer data )
 {
   GList *iter = val->children;
   VikLayer *vl;
-  VikLayer *trigger = VIK_LAYER(vik_viewport_get_trigger( VIK_VIEWPORT(data) ));
+  VikViewport *viewport = VIK_VIEWPORT(data);
+  VikLayer *trigger = VIK_LAYER(vik_viewport_get_trigger( viewport ));
   while ( iter ) {
     vl = VIK_LAYER(iter->data);
     if ( vl == trigger ) {
-      if ( vik_viewport_get_half_drawn ( VIK_VIEWPORT(data) ) ) {
-        vik_viewport_set_half_drawn ( VIK_VIEWPORT(data), FALSE );
-        vik_viewport_snapshot_load( VIK_VIEWPORT(data) );
+      if ( vik_viewport_get_half_drawn ( viewport ) ) {
+        vik_viewport_set_half_drawn ( viewport, FALSE );
+        vik_viewport_snapshot_load( viewport );
       } else {
-        vik_viewport_snapshot_save( VIK_VIEWPORT(data) );
+        vik_viewport_snapshot_save( viewport );
       }
     }
-    if ( vl->type == VIK_LAYER_AGGREGATE || vl->type == VIK_LAYER_GPS || ! vik_viewport_get_half_drawn( VIK_VIEWPORT(data) ) )
+    if ( vl->type == VIK_LAYER_AGGREGATE || vl->type == VIK_LAYER_GPS || ! vik_viewport_get_half_drawn( viewport ) )
       vik_layer_draw ( vl, data );
     iter = iter->next;
   }
@@ -393,11 +406,12 @@ VikLayer *vik_aggregate_layer_get_top_visible_layer_of_type ( VikAggregateLayer
 
   while ( ls )
   {
-    if ( VIK_LAYER(ls->data)->visible && VIK_LAYER(ls->data)->type == type )
-      return VIK_LAYER(ls->data);
-    else if ( VIK_LAYER(ls->data)->visible && VIK_LAYER(ls->data)->type == VIK_LAYER_AGGREGATE )
+    VikLayer *vl = VIK_LAYER(ls->data);
+    if ( vl->visible && vl->type == type )
+      return vl;
+    else if ( vl->visible && vl->type == VIK_LAYER_AGGREGATE )
     {
-      rv = vik_aggregate_layer_get_top_visible_layer_of_type(VIK_AGGREGATE_LAYER(ls->data), type);
+      rv = vik_aggregate_layer_get_top_visible_layer_of_type(VIK_AGGREGATE_LAYER(vl), type);
       if ( rv )
         return rv;
     }
@@ -406,17 +420,48 @@ VikLayer *vik_aggregate_layer_get_top_visible_layer_of_type ( VikAggregateLayer
   return NULL;
 }
 
-GList *vik_aggregate_layer_get_all_layers_of_type(VikAggregateLayer *val, GList *layers, gint type)
+GList *vik_aggregate_layer_get_all_layers_of_type(VikAggregateLayer *val, GList *layers, gint type, gboolean include_invisible)
 {
   GList *l = layers;
   GList *children = val->children;
+  VikLayer *vl;
   if (!children)
     return layers;
+
+  // Where appropriate *don't* include non-visible layers
   while (children) {
-    if (VIK_LAYER(children->data)->type == VIK_LAYER_AGGREGATE)
-      l = vik_aggregate_layer_get_all_layers_of_type(VIK_AGGREGATE_LAYER(children->data), l, type); 
-    else if (VIK_LAYER(children->data)->type == type)
-      l = g_list_prepend(l, children->data); /* now in top down order */
+    vl = VIK_LAYER(children->data);
+    if (vl->type == VIK_LAYER_AGGREGATE ) {
+      // Don't even consider invisible aggregrates, unless told to
+      if (vl->visible || include_invisible)
+	l = vik_aggregate_layer_get_all_layers_of_type(VIK_AGGREGATE_LAYER(children->data), l, type, include_invisible);
+    }
+    else if (vl->type == type) {
+      if (vl->visible || include_invisible)
+	l = g_list_prepend(l, children->data); /* now in top down order */
+    }
+    else if (type == VIK_LAYER_TRW) {
+      /* GPS layers contain TRW layers. cf with usage in file.c */
+      if (VIK_LAYER(children->data)->type == VIK_LAYER_GPS) {
+	if (VIK_LAYER(children->data)->visible || include_invisible) {
+	  if (!vik_gps_layer_is_empty(VIK_GPS_LAYER(children->data))) {
+	    /*
+	      can not use g_list_concat due to wrong copy method - crashes if used a couple times !!
+	      l = g_list_concat (l, vik_gps_layer_get_children (VIK_GPS_LAYER(children->data)));
+	    */
+	    /* create own copy method instead :( */
+	    GList *gps_trw_layers = (GList *)vik_gps_layer_get_children (VIK_GPS_LAYER(children->data));
+	    int n_layers = g_list_length (gps_trw_layers);
+	    int layer = 0;
+	    for ( layer = 0; layer < n_layers; layer++) {
+	      l = g_list_prepend(l, gps_trw_layers->data);
+	      gps_trw_layers = gps_trw_layers->next;
+	    }
+	    g_list_free(gps_trw_layers);
+	  }
+	}
+      }
+    }
     children = children->next;
   }
   return l;
@@ -426,13 +471,16 @@ void vik_aggregate_layer_realize ( VikAggregateLayer *val, VikTreeview *vt, GtkT
 {
   GList *i = val->children;
   GtkTreeIter iter;
+  VikLayer *vl = VIK_LAYER(val);
+  VikLayer *vli;
   while ( i )
   {
-    vik_treeview_add_layer ( VIK_LAYER(val)->vt, layer_iter, &iter, VIK_LAYER(i->data)->name, val, 
-        VIK_LAYER(i->data), VIK_LAYER(i->data)->type, VIK_LAYER(i->data)->type );
-    if ( ! VIK_LAYER(i->data)->visible )
-      vik_treeview_item_set_visible ( VIK_LAYER(val)->vt, &iter, FALSE );
-    vik_layer_realize ( VIK_LAYER(i->data), VIK_LAYER(val)->vt, &iter );
+    vli = VIK_LAYER(i->data);
+    vik_treeview_add_layer ( vl->vt, layer_iter, &iter, vli->name, val, 
+        vli, vli->type, vli->type );
+    if ( ! vli->visible )
+      vik_treeview_item_set_visible ( vl->vt, &iter, FALSE );
+    vik_layer_realize ( vli, vl->vt, &iter );
     i = i->next;
   }
 }
