@@ -28,6 +28,7 @@
 #include <string.h>
 
 #include <glib/gi18n.h>
+#include <gdk/gdkkeysyms.h>
 
 enum {
   VLP_UPDATE_SIGNAL,
@@ -69,6 +70,7 @@ static void menu_popup_cb (VikLayersPanel *vlp);
 static void layers_popup_cb (VikLayersPanel *vlp);
 static void layers_popup ( VikLayersPanel *vlp, GtkTreeIter *iter, gint mouse_button );
 static gboolean layers_button_press_cb (VikLayersPanel *vlp, GdkEventButton *event);
+static gboolean layers_key_press_cb (VikLayersPanel *vlp, GdkEventKey *event);
 static void layers_move_item ( VikLayersPanel *vlp, gboolean up );
 static void layers_move_item_up ( VikLayersPanel *vlp );
 static void layers_move_item_down ( VikLayersPanel *vlp );
@@ -157,6 +159,7 @@ static void layers_panel_init ( VikLayersPanel *vlp )
   g_signal_connect_swapped ( vlp->vt, "button_press_event", G_CALLBACK(layers_button_press_cb), vlp);
   g_signal_connect_swapped ( vlp->vt, "item_toggled", G_CALLBACK(layers_item_toggled), vlp);
   g_signal_connect_swapped ( vlp->vt, "item_edited", G_CALLBACK(layers_item_edited), vlp);
+  g_signal_connect_swapped ( vlp->vt, "key_press_event", G_CALLBACK(layers_key_press_cb), vlp);
 
   /* Add button */
   addimage = gtk_image_new_from_stock ( GTK_STOCK_ADD, GTK_ICON_SIZE_SMALL_TOOLBAR );
@@ -221,7 +224,10 @@ static void layers_panel_init ( VikLayersPanel *vlp )
   {
     /* TODO: FIXME: if name has a '/' in it it will get all messed up. why not have an itemfactory field with
                     name, icon, shortcut, etc.? */
-    entry.path = g_strdup_printf("%s/New %s Layer", base_entries[NUM_BASE_ENTRIES-1].path, vik_layer_get_interface(i)->name );
+    /* Note: we use a temporary label in order to share translation with other codde */
+    gchar *label = g_strdup_printf(_("New %s Layer"), vik_layer_get_interface(i)->name );
+    entry.path = g_strdup_printf("%s/%s", base_entries[NUM_BASE_ENTRIES-1].path, label );
+    g_free ( label );
     entry.accelerator = NULL;
     entry.callback = (GtkItemFactoryCallback) vik_layers_panel_new_layer;
     entry.callback_action = i;
@@ -311,6 +317,16 @@ static gboolean layers_button_press_cb ( VikLayersPanel *vlp, GdkEventButton *ev
   return FALSE;
 }
 
+static gboolean layers_key_press_cb ( VikLayersPanel *vlp, GdkEventKey *event )
+{
+  // Accept all forms of delete keys
+  if (event->keyval == GDK_Delete || event->keyval == GDK_KP_Delete || event->keyval == GDK_BackSpace) {
+    vik_layers_panel_delete_selected (vlp);
+    return TRUE;
+ }
+ return FALSE;
+}
+
 static void layers_popup ( VikLayersPanel *vlp, GtkTreeIter *iter, gint mouse_button )
 {
   GtkMenu *menu = NULL;
@@ -372,7 +388,7 @@ static void layers_popup ( VikLayersPanel *vlp, GtkTreeIter *iter, gint mouse_bu
     else
     {
       menu = GTK_MENU ( gtk_menu_new () );
-      if ( ! vik_layer_sublayer_add_menu_items ( vik_treeview_item_get_parent ( vlp->vt, iter ), menu, vlp, vik_treeview_item_get_data ( vlp->vt, iter ), vik_treeview_item_get_pointer ( vlp->vt, iter ), iter ) )
+      if ( ! vik_layer_sublayer_add_menu_items ( vik_treeview_item_get_parent ( vlp->vt, iter ), menu, vlp, vik_treeview_item_get_data ( vlp->vt, iter ), vik_treeview_item_get_pointer ( vlp->vt, iter ), iter, vlp->vvp ) )
       {
         gtk_widget_destroy ( GTK_WIDGET(menu) );
         return;
@@ -468,7 +484,7 @@ static void layers_move_item ( VikLayersPanel *vlp, gboolean up )
   if ( ! vik_treeview_get_selected_iter ( vlp->vt, &iter ) )
     return;
 
-  vik_treeview_select_iter ( vlp->vt, &iter ); /* cancel any layer-name editing going on... */
+  vik_treeview_select_iter ( vlp->vt, &iter, FALSE ); /* cancel any layer-name editing going on... */
 
   if ( vik_treeview_item_get_type ( vlp->vt, &iter ) == VIK_TREEVIEW_TYPE_LAYER )
   {
@@ -516,7 +532,6 @@ void vik_layers_panel_cut_selected ( VikLayersPanel *vlp )
   gint type;
   GtkTreeIter iter;
   
-  g_debug(__FUNCTION__);
   if ( ! vik_treeview_get_selected_iter ( vlp->vt, &iter ) )
     /* Nothing to do */
     return;
@@ -526,37 +541,39 @@ void vik_layers_panel_cut_selected ( VikLayersPanel *vlp )
   if ( type == VIK_TREEVIEW_TYPE_LAYER )
   {
     VikAggregateLayer *parent = vik_treeview_item_get_parent ( vlp->vt, &iter );
-
     if ( parent )
     {
-
       /* reset trigger if trigger deleted */
       if ( vik_layers_panel_get_selected ( vlp ) == vik_viewport_get_trigger ( vlp->vvp ) )
         vik_viewport_set_trigger ( vlp->vvp, NULL );
 
       a_clipboard_copy_selected ( vlp );
-      if ( vik_aggregate_layer_delete ( parent, &iter ) )
-        vik_layers_panel_emit_update ( vlp );
+
+      if (IS_VIK_AGGREGATE_LAYER(parent)) {
+	if ( vik_aggregate_layer_delete ( parent, &iter ) )
+	  vik_layers_panel_emit_update ( vlp );
+      }
     }
     else
       a_dialog_info_msg ( VIK_GTK_WINDOW_FROM_WIDGET(vlp), _("You cannot cut the Top Layer.") );
   }
+  else if (type == VIK_TREEVIEW_TYPE_SUBLAYER) {
+    VikLayer *sel = vik_layers_panel_get_selected ( vlp );
+    if ( vik_layer_get_interface(sel->type)->cut_item ) {
+      gint subtype = vik_treeview_item_get_data( vlp->vt, &iter);
+      vik_layer_get_interface(sel->type)->cut_item ( sel, subtype, vik_treeview_item_get_pointer(sel->vt, &iter) );
+    }
+  }
 }
 
 void vik_layers_panel_copy_selected ( VikLayersPanel *vlp )
 {
-  gint type;
   GtkTreeIter iter;
-  
   if ( ! vik_treeview_get_selected_iter ( vlp->vt, &iter ) )
     /* Nothing to do */
     return;
-
-  type = vik_treeview_item_get_type ( vlp->vt, &iter );
-
-  if ( type == VIK_TREEVIEW_TYPE_LAYER ) {
-    a_clipboard_copy_selected ( vlp );
-  }
+  // NB clipboard contains layer vs sublayer logic, so don't need to do it here
+  a_clipboard_copy_selected ( vlp );
 }
 
 void vik_layers_panel_paste_selected ( VikLayersPanel *vlp )
@@ -581,6 +598,12 @@ void vik_layers_panel_delete_selected ( VikLayersPanel *vlp )
 
   if ( type == VIK_TREEVIEW_TYPE_LAYER )
   {
+    // Get confirmation from the user
+    if ( ! a_dialog_yes_or_no ( VIK_GTK_WINDOW_FROM_WIDGET(vlp),
+				_("Are you sure you want to delete %s?"),
+				vik_layer_get_name ( VIK_LAYER(vik_treeview_item_get_pointer ( vlp->vt, &iter )) ) ) )
+      return;
+
     VikAggregateLayer *parent = vik_treeview_item_get_parent ( vlp->vt, &iter );
     if ( parent )
     {
@@ -664,11 +687,11 @@ VikLayer *vik_layers_panel_get_layer_of_type ( VikLayersPanel *vlp, gint type )
     return rv;
 }
 
-GList *vik_layers_panel_get_all_layers_of_type(VikLayersPanel *vlp, gint type)
+GList *vik_layers_panel_get_all_layers_of_type(VikLayersPanel *vlp, gint type, gboolean include_invisible)
 {
   GList *layers = NULL;
 
-  return (vik_aggregate_layer_get_all_layers_of_type ( vlp->toplayer, layers, type ));
+  return (vik_aggregate_layer_get_all_layers_of_type ( vlp->toplayer, layers, type, include_invisible));
 }
 
 VikAggregateLayer *vik_layers_panel_get_top_layer ( VikLayersPanel *vlp )
@@ -678,7 +701,7 @@ VikAggregateLayer *vik_layers_panel_get_top_layer ( VikLayersPanel *vlp )
 
 void vik_layers_panel_clear ( VikLayersPanel *vlp )
 {
-  if ( (! vik_aggregate_layer_is_empty(vlp->toplayer)) && a_dialog_overwrite ( VIK_GTK_WINDOW_FROM_WIDGET(vlp), _("Are you sure you wish to delete all layers?"), NULL ) )
+  if ( (! vik_aggregate_layer_is_empty(vlp->toplayer)) && a_dialog_yes_or_no ( VIK_GTK_WINDOW_FROM_WIDGET(vlp), _("Are you sure you wish to delete all layers?"), NULL ) )
     vik_aggregate_layer_clear ( vlp->toplayer ); /* simply deletes all layers */
 }
 
@@ -695,3 +718,7 @@ static void layers_panel_finalize ( GObject *gob )
   G_OBJECT_CLASS(parent_class)->finalize(gob);
 }
 
+VikTreeview *vik_layers_panel_get_treeview ( VikLayersPanel *vlp )
+{
+  return vlp->vt;
+}
