@@ -25,7 +25,7 @@
 
 #include <glib.h>
 #include <glib-object.h>
-#include <gtk/gtkdrawingarea.h>
+#include <gtk/gtk.h>
 
 #include "vikcoord.h"
 
@@ -98,6 +98,7 @@ typedef enum {
   VIK_VIEWPORT_DRAWMODE_UTM=0,
   VIK_VIEWPORT_DRAWMODE_EXPEDIA,
   VIK_VIEWPORT_DRAWMODE_MERCATOR,
+  VIK_VIEWPORT_DRAWMODE_LATLON,
   VIK_VIEWPORT_NUM_DRAWMODES      /*< skip >*/
 } VikViewportDrawMode;
 
@@ -134,13 +135,23 @@ void vik_viewport_draw_pixbuf ( VikViewport *vvp, GdkPixbuf *pixbuf, gint src_x,
 gint vik_viewport_get_width ( VikViewport *vvp );
 gint vik_viewport_get_height ( VikViewport *vvp );
 
+void vik_viewport_reset_copyrights ( VikViewport *vp );
+void vik_viewport_add_copyright ( VikViewport *vp, const gchar *copyright );
+
+void vik_viewport_reset_logos ( VikViewport *vp );
+void vik_viewport_add_logo ( VikViewport *vp, const GdkPixbuf *logo );
+
 /* Viewport features */
 void vik_viewport_draw_scale ( VikViewport *vvp );
 void vik_viewport_set_draw_scale ( VikViewport *vvp, gboolean draw_scale );
 gboolean vik_viewport_get_draw_scale ( VikViewport *vvp );
+void vik_viewport_draw_copyright ( VikViewport *vvp );
 void vik_viewport_draw_centermark ( VikViewport *vvp );
 void vik_viewport_set_draw_centermark ( VikViewport *vvp, gboolean draw_centermark );
 gboolean vik_viewport_get_draw_centermark ( VikViewport *vvp );
+void vik_viewport_draw_logo ( VikViewport *vvp );
+void vik_viewport_set_draw_highlight ( VikViewport *vvp, gboolean draw_highlight );
+gboolean vik_viewport_get_draw_highlight ( VikViewport *vvp );
 
 /* Color/graphics context management */
 void vik_viewport_set_background_color ( VikViewport *vvp, const gchar *color );
@@ -152,6 +163,12 @@ GdkGC *vik_viewport_new_gc ( VikViewport *vvp, const gchar *colorname, gint thic
 GdkGC *vik_viewport_new_gc_from_color ( VikViewport *vvp, GdkColor *color, gint thickness );
 GdkFunction vik_gc_get_function ( GdkGC *gc );
 
+void vik_viewport_set_highlight_color ( VikViewport *vvp, const gchar *color );
+const gchar *vik_viewport_get_highlight_color ( VikViewport *vvp );
+GdkColor *vik_viewport_get_highlight_gdkcolor ( VikViewport *vvp );
+void vik_viewport_set_highlight_gdkcolor ( VikViewport *vvp, GdkColor * );
+GdkGC* vik_viewport_get_gc_highlight ( VikViewport *vvp );
+
 /* Drawing primitives */
 void a_viewport_clip_line ( gint *x1, gint *y1, gint *x2, gint *y2 ); /* run this before drawing a line. vik_viewport_draw_line runs it for you */
 void vik_viewport_draw_line ( VikViewport *vvp, GdkGC *gc, gint x1, gint y1, gint x2, gint y2 );
